
import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;
import jade.domain.*;
import jade.domain.FIPAAgentManagement.*;

public class BookBuyerAgent extends Agent {
	// Nome do livro a ser comprado por esse agente
	private String bookTitle;
	// Lista dos agentes vendedores a ser obtida a parir do DF
	private AID[] sellerAgents;

	protected void setup() {
		System.out.println("Ola! Agente Comprador " + getAID().getName() + " esta pronto!");

		// Obtem o nome do livro a ser comprado
		Object[] args = getArguments();
		if (args != null && args.length > 0) {
			bookTitle = (String) args[0];
			System.out.println("Tentando comprar o livro " + bookTitle);

                        // Devido a esse comportamento o agente tentara comprar o livros a cada 30seg
			addBehaviour( new TickerBehaviour (this, 30000) {
				protected void onTick() {

					// Obtem o AID dos agentes que vendem livros fazendo uma solicitao ao DF
					DFAgentDescription template = new DFAgentDescription();
					ServiceDescription sd = new ServiceDescription();
					// Busca pelos agentes que oferedem o servio do tipo book-selling
					sd.setType("book-selling");
					template.addServices(sd);
					try {
						DFAgentDescription[] result = DFService.search(myAgent, template);
						sellerAgents = new AID[result.length];
						for (int i = 0; i < result.length; i++) {
							sellerAgents[i] = result[i].getName();
						}
					}
					catch (FIPAException fe) {
						fe.printStackTrace();
					}

					// Adiciona o comportamento de compra
					addBehaviour(new RequestPerformer());
				}
			}
			);
		}
	}

 	protected void takeDown() {
		System.out.println("Agente comprador " + getAID().getName() + " concludo!");
	}
	
	private class RequestPerformer extends Behaviour {
		private AID bestSeller; // Agente que forneceu a melhor oferta
		private int bestPrice; // Valor da melhor oferta
		private int repliesCnt = 0; // Contador de respostas recebidas
		private MessageTemplate mt; // Template utilziado para receber as entradas
		private int step = 0;

		public void action() {
			switch (step) {
			case 0:
				// Envia uma mensagem de CFP para todos so agentes vendedores
				ACLMessage cfp = new ACLMessage(ACLMessage.CFP);
				for (int i = 0; i < sellerAgents.length; ++i) {
					 cfp.addReceiver(sellerAgents[i]);
				}
				cfp.setContent(bookTitle);
				cfp.setConversationId("book-trade");
				cfp.setReplyWith("cfp"+System.currentTimeMillis()); // Valor unico
				myAgent.send(cfp);
				// Prepara o template para o recebimento das respostas dos agentes vendedores
				mt = MessageTemplate.and(MessageTemplate.MatchConversationId("book-trade"),
							 MessageTemplate.MatchInReplyTo(cfp.getReplyWith()));
				step = 1;
				break;
			case 1:
				// Recebe todo as prpostas/recusas dos agentes vendedores
				ACLMessage reply = myAgent.receive(mt);
				if (reply != null) { // Resposta recebida
					if (reply.getPerformative() == ACLMessage.PROPOSE) {  // Uma oferta foi recebida
						 int price = Integer.parseInt(reply.getContent());
						 if (bestSeller == null || price < bestPrice) { // identifica a melhor oferta
							  bestPrice = price;
							  bestSeller = reply.getSender();
						 }
					}
					repliesCnt++;
					if (repliesCnt >= sellerAgents.length) // Todas as respostas foram recebidas
						 step = 2;
				}
				else block();
				break;
			case 2:
				// Envia a solicitao do pedidio para o agente com a melhor proposta
				ACLMessage order = new ACLMessage(ACLMessage.ACCEPT_PROPOSAL);
				order.addReceiver(bestSeller);
				order.setContent(bookTitle);
				order.setConversationId("book-trade");
				order.setReplyWith("order"+System.currentTimeMillis());
				myAgent.send(order);
				// Prepara o template para receber a resposta do pedido de compra
				mt = MessageTemplate.and(MessageTemplate.MatchConversationId("book-trade"),
						  				 MessageTemplate.MatchInReplyTo(order.getReplyWith()));
				step = 3;
				break;
			case 3:
				// Recebe a resposta do pedido de compra
				reply = myAgent.receive(mt);
				if (reply != null) {
					 // Resposta do pedidio de compra recebido
					 if (reply.getPerformative() == ACLMessage.INFORM) {
						  // Compra com sucesso. Agente pode ser encerrado
						  System.out.println(bookTitle+" adquirido com sucesso.");
						  System.out.println("Preo:  "+bestPrice);
						  myAgent.doDelete();
					 }
					 step = 4;
				}
				else block();
				break;
			}
		}

		public boolean done() {
			return ( (step == 2 && bestSeller == null) || (step == 4));
		}
	}

}
