
import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;
import jade.domain.*;
import jade.domain.FIPAAgentManagement.*;

public class BookSellerAgent extends Agent {
	private String bookTitle; // Nome do livro para ser vendido
	private int bookPrice; // Valor do livro

	protected void setup() {
		System.out.println("Ola! Agente vendedor " + getAID().getName() + " esta pronto!");
		// Obtem o nome do livro a ser vendido e o seu vlor
		Object[] args = getArguments();
		if (args != null && args.length > 0) {
			bookTitle = (String) args[0];
			bookPrice = Integer.parseInt((String)args[1]);
			System.out.println("Vendendo o livro " + bookTitle + " por R$ " + bookPrice);

                        // Prepara uma descrio do agente que contera os servios que o mesmo disponibilza
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			ServiceDescription sd = new ServiceDescription();
			sd.setType("book-selling");
			sd.setName("JADE-book-trading");
			// Adiciona o servio a descrio do agente
			dfd.addServices(sd);
			try {
                                // Registra o servio justo ao DF
				DFService.register(this, dfd);
			}
			catch (FIPAException fe) {
				fe.printStackTrace();
			}

                        // Adiciona comportamento para oferecer propostas
			addBehaviour(new OfferRequestsServer());
			// Adiciona comportamento para lidar com pedidos de compra
			addBehaviour(new PurchaseOrderServer());
		}
	}

	protected void takeDown() {
		System.out.println("Agente vendedor " + getAID().getName() + " concludo!");
	}

        // Comportamento ciclico, ou seja, estara sempre disponivel para execuo enquanto o agente estiver ativo
	private class OfferRequestsServer extends CyclicBehaviour {
		public void action() {
                        // Prepara template para recebimento de mensagens de mensagens CFP
			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchConversationId("book-trade"),
						 MessageTemplate.MatchPerformative(ACLMessage.CFP));
			ACLMessage msg = receive(mt);
			if (msg != null) {
				String title = msg.getContent();
				ACLMessage reply = msg.createReply();

                                // Testa se o livro que o agente compraddor deseja obter eh o mesmo que ele possui
				if (bookTitle.equals(title)) {
                                        // Prepara um proposta para enviar ao agente comprador
					reply.setPerformative(ACLMessage.PROPOSE);
					reply.setContent(String.valueOf(bookPrice));
				}
				else {
                                        // Caso o agente nao possua o livro, envia uma responda com performativa REFUSE
					reply.setPerformative(ACLMessage.REFUSE);
					reply.setContent("not-available");
				}
				send(reply);
			}
			else block();
		}
	}

        // Comportamento ciclico, ou seja, estara sempre disponivel para execuo enquanto o agente estiver ativo
	private class PurchaseOrderServer extends CyclicBehaviour {

		public void action() {
                        // Prepara template para recebimento de mensagens de mensagens ACCEPT-PROPOSAL
			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchConversationId("book-trade"),
									 MessageTemplate.MatchPerformative(ACLMessage.ACCEPT_PROPOSAL));

			ACLMessage msg = receive(mt);

			if (msg != null) {
                                // Prepara uma mensagem informando apenas que o livro foi vendido com sucesso
				ACLMessage reply = msg.createReply();
				reply.setPerformative(ACLMessage.INFORM);
				reply.setContent("book-sold");
				send(reply);
			}
			else block();
		}
	}

}
